SystemOrganization addCategory: #'Grease-Pharo-Core'!

GRCodec subclass: #GRPharoGenericCodec
	instanceVariableNames: 'name urlCodec'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Grease-Pharo-Core'!

!GRPharoGenericCodec class methodsFor: 'private' stamp: 'pmm 2/20/2009 13:23'!
additionalLatin1CodecNames
	^#('iso-8859-1' 'ISO-8859-1')! !

!GRPharoGenericCodec class methodsFor: 'private' stamp: 'pmm 6/28/2009 15:43'!
basicForEncoding: aString
	(self supportsEncoding: aString)
		ifFalse: [ self unsupportedEncoding: aString ].
	^ self basicNew initializeWithName: aString! !

!GRPharoGenericCodec class methodsFor: 'accessing' stamp: 'jf 9/30/2009 01:10'!
codecs
	^ (TextConverter allEncodingNames select: 
		[ :each | 
		"exclude UFT-8 which is handeled by WAUtf8Codec"
		self supportsEncoding: each ]) collect: [ :each | self basicForEncoding: each greaseString ]! !

!GRPharoGenericCodec class methodsFor: 'private' stamp: 'pmm 2/20/2009 14:26'!
latin1CodecNames
	^Latin1TextConverter encodingNames, self additionalLatin1CodecNames! !

!GRPharoGenericCodec class methodsFor: 'private' stamp: 'pmm 6/7/2009 14:14'!
supportedEncodingNames
	"answers the names of the encodings supported by this class"
	^TextConverter allEncodingNames
		addAll: self additionalLatin1CodecNames;
		removeAllFoundIn: UTF8TextConverter encodingNames;
		yourself! !

!GRPharoGenericCodec class methodsFor: 'testing' stamp: 'pmm 6/28/2009 16:43'!
supportsEncoding: aString
	"Answer whether the the given encoding name is supported."
	^ self supportedEncodingNames includes: aString! !

!GRPharoGenericCodec methodsFor: 'private' stamp: 'pmm 2/20/2009 14:27'!
converter
	^(self class latin1CodecNames includes: self name)
		ifTrue: [ Latin1TextConverter new ]
		ifFalse: [ TextConverter newForEncoding: self name ]! !

!GRPharoGenericCodec methodsFor: 'conversion' stamp: 'lr 2/7/2009 12:52'!
decoderFor: aStream
	^ self encoderFor: aStream! !

!GRPharoGenericCodec methodsFor: 'conversion' stamp: 'jf 9/30/2009 00:46'!
encoderFor: aStream 
	^ GRPharoConverterCodecStream 
		on: aStream
		converter: self converter! !

!GRPharoGenericCodec methodsFor: 'initialization' stamp: 'jf 9/30/2009 00:27'!
initializeWithName: aString 
	self initialize.
	name := aString.
	urlCodec := (self class latin1CodecNames includes: aString) 
		ifFalse: [ GRCodec forEncoding: 'utf-8' ]
		ifTrue: [ self ]! !

!GRPharoGenericCodec methodsFor: 'accessing' stamp: 'lr 2/7/2009 10:16'!
name
	^ name! !

!GRPharoGenericCodec methodsFor: 'accessing' stamp: 'lr 2/7/2009 12:36'!
url
	"RFC 3986: When a new URI scheme defines a component that represents textual data consisting of characters from the Universal Character Set [UCS], the data should first be encoded as octets according to the UTF-8 character encoding."

	^ urlCodec! !

GRCodec subclass: #GRPharoUtf8Codec
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Grease-Pharo-Core'!

!GRPharoUtf8Codec commentStamp: 'pmm 2/20/2009 12:51' prior: 0!
A WAUtf8Codec is a WACodec optimized for UTF-8.!

!GRPharoUtf8Codec class methodsFor: 'private' stamp: 'pmm 6/28/2009 15:43'!
basicForEncoding: aString
	^self new! !

!GRPharoUtf8Codec class methodsFor: 'accessing' stamp: 'pmm 7/18/2009 18:13'!
codecs
	^Array with: self new! !

!GRPharoUtf8Codec class methodsFor: 'testing' stamp: 'pmm 11/15/2009 11:02'!
supportsEncoding: aString
	^(#('utf-8' 'UTF-8') includes: aString)
		or: [ UTF8TextConverter encodingNames includes: aString ]! !

!GRPharoUtf8Codec methodsFor: 'decoding' stamp: 'pmm 2/20/2009 16:00'!
decode: aString
	| readStream writeStream |
	readStream := self decoderFor: aString asByteArray readStream.
	writeStream := WriteStream on: (String new: aString size).
	[ readStream atEnd ]
		whileFalse: [ writeStream nextPutAll: (readStream next: 16rFFF) ].
	^ writeStream contents! !

!GRPharoUtf8Codec methodsFor: 'conversion' stamp: 'jf 9/30/2009 00:46'!
decoderFor: aStream 
	^ GRPharoUtf8CodecStream 
		on: aStream
		converter: UTF8TextConverter new! !

!GRPharoUtf8Codec methodsFor: 'conversion' stamp: 'jf 9/30/2009 00:46'!
encoderFor: aStream 
	^ GRPharoUtf8CodecStream 
		on: aStream
		converter: UTF8TextConverter new! !

!GRPharoUtf8Codec methodsFor: 'accessing' stamp: 'pmm 2/20/2009 13:25'!
name
	^ 'utf-8'! !

!GRPharoUtf8Codec methodsFor: 'accessing' stamp: 'pmm 2/20/2009 12:50'!
url
	^ self! !

!MessageSend methodsFor: '*grease-pharo-core' stamp: 'pmm 1/10/2009 19:22'!
argumentCount
	^ selector numArgs - self arguments size! !

!MessageSend methodsFor: '*grease-pharo-core' stamp: 'pmm 4/23/2007 20:02'!
fixCallbackTemps
	"for polymorphism with BlockContext >> #fixCallbackTemps"! !

!MessageSend methodsFor: '*grease-pharo-core' stamp: 'ab 3/5/2003 18:30'!
numArgs
	^ selector numArgs! !

!MessageSend methodsFor: '*grease-pharo-core' stamp: 'pmm 1/10/2009 18:40'!
value: anObject
	^ self valueWithArguments: (Array with: anObject)! !

!MessageSend methodsFor: '*grease-pharo-core' stamp: 'pmm 1/10/2009 23:54'!
value: aFirstObject value: aSecondObject
	^ self valueWithArguments: (Array with: aFirstObject with: aSecondObject)! !

!MessageSend methodsFor: '*grease-pharo-core' stamp: 'pmm 1/10/2009 23:53'!
valueWithPossibleArguments: anArray

     "Evaluate the block represented by the receiver.
     If the block requires one argument, use anArg, if it requires more than one,
     fill up the rest with nils."

	^self valueWithEnoughArguments: anArray! !

!Behavior methodsFor: '*grease-pharo-core' stamp: 'pmm 10/30/2006 18:02'!
fullName
	"In VW, will include the namespace"
	^self name! !

!BlockContext methodsFor: '*grease-pharo-core' stamp: 'lr 10/28/2007 14:42'!
fixCallbackTemps
	"Fix the values of the temporary variables used in the block that are
	ordinarily shared with the method in which the block is defined.

	This is needed because Squeak is a crappy Smalltalk implementation
	that does not have full blcck closures."

	| temps |
	home := home copy.
	home swapSender: nil.
	home isMethodContext
		ifFalse: [ ^ self ].
	temps := self tempVarRefs.
	1 to: home size do: [ :index |
		(temps includes: index)
			ifFalse: [ home tempAt: index put: nil ] ]! !

!BlockContext methodsFor: '*grease-pharo-core' stamp: 'lr 8/25/2007 16:26'!
tempVarRefs
	| method scanner end tempVars byte type offset |
	home ifNil: [ ^ false ].
	tempVars := Set new.
	method := self method.
	"Determine end of block from long jump preceding it"
	end := (method at: startpc - 2) \\ 16 - 4 * 256 + (method at: startpc - 1) + startpc - 1.
	scanner := InstructionStream new method: method pc: startpc.
	[ scanner pc <= end ] whileTrue: [
		byte  := scanner nextByte.
		type := byte // 16.
		offset := byte \\ 16.
		type = 1 ifTrue: [ tempVars add: offset + 1 ].
		scanner nextInstruction ].
	^ tempVars! !

!BlockContext methodsFor: '*grease-pharo-core' stamp: 'pmm 1/10/2009 23:42'!
valueWithPossibleArguments: anArray
	^self valueWithEnoughArguments: anArray! !

GRPlatform subclass: #GRPharoPlatform
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Grease-Pharo-Core'!

!GRPharoPlatform commentStamp: 'pmm 6/1/2008 01:03' prior: 0!
A WASqueakPlatform is the Squeak implementation of SeasidePlatformSupport, the Seaside class that provides functionality that can not be implemented in a platform independent way.
!

!GRPharoPlatform class methodsFor: 'class initialization' stamp: 'jf 1/22/2009 02:45'!
initialize
	self select! !

!GRPharoPlatform class methodsFor: 'class initialization' stamp: 'jf 1/22/2009 02:45'!
unload
	self unselect! !

!GRPharoPlatform methodsFor: 'startup' stamp: 'jf 1/21/2009 17:31'!
addToShutDownList: anObject
	"Add anObject to the shutdown-list of the system. On shutdown the message #shutDown will be sent to anObject."

	Smalltalk addToShutDownList: anObject! !

!GRPharoPlatform methodsFor: 'startup' stamp: 'jf 1/21/2009 17:31'!
addToStartUpList: anObject
	"Add anObject to the startup-list of the system. On startup the message #startUp will be sent to anObject."

	Smalltalk addToStartUpList: anObject! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'jf 1/21/2009 17:31'!
asMethodReturningByteArray: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a ByteArray"
	^self useByteArrayLiterals
		ifTrue: [ self asMethodReturningByteArrayLiteral: aByteArrayOrString named: aSymbol ]
		ifFalse: [ self asMethodReturningByteArrayWithCache: aByteArrayOrString named: aSymbol ]! !

!GRPharoPlatform methodsFor: 'private-file library' stamp: 'lr 2/4/2009 11:28'!
asMethodReturningByteArrayLiteral: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a byte array using VW/NewCompiler byte array literal syntax."
	^String streamContents: [ :stream |
		stream nextPutAll: aSymbol; nextPut: Character cr.
		stream tab; nextPutAll: '^ #['.
		aByteArrayOrString asByteArray
			do: [ :each | each printOn: stream ]
			separatedBy: [ stream space ].
		stream nextPutAll: ']' ]! !

!GRPharoPlatform methodsFor: 'private-file library' stamp: 'pmm 2/20/2009 17:44'!
asMethodReturningByteArrayWithCache: aByteArrayOrString named: aSymbol
	"Generates the source of a method named aSymbol that returns aByteArrayOrString as a byte array and caching this array in a literal array of size 1."
	^ String streamContents: [ :stream |
		stream nextPutAll: aSymbol; nextPut: Character cr.
		stream tab; nextPutAll: '^ #('.
		aByteArrayOrString asByteArray
			do: [ :each | each printOn: stream ]
			separatedBy: [ stream space ].
		stream nextPutAll: ') asByteArray' ]! !

!GRPharoPlatform methodsFor: 'encoding' stamp: 'jf 1/21/2009 17:31'!
base64Decode: aString
	^ (Base64MimeConverter mimeDecodeToChars: aString readStream) contents! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'jf 9/30/2009 00:20'!
compile: aString into: aClass classified: aSymbol 
	"The trick here is to be as silently a possible so that the package is not marked dirty when running WAFileLibrary test.
	This also makes running tests much faster."
	| methodNode compiledMethod selector methodAndNode |
	methodNode := aClass compilerClass new 
		compile: aString
		in: aClass
		notifying: nil
		ifFail: [ GRError signal: 'syntax error' ].
	selector := methodNode selector.
	methodAndNode := CompiledMethodWithNode 
		generateMethodFromNode: methodNode
		trailer: aClass defaultMethodTrailer.
	compiledMethod := methodAndNode method.
	compiledMethod 
		putSource: aString
		fromParseNode: methodAndNode node
		inFile: 2
		withPreamble: 
			[ :file | 
			file
				nextPut: Character cr;
				nextPut: $!!;
				nextChunkPut: 'Behavior method';
				nextPut: Character cr ].
	aClass 
		addSelectorSilently: selector
		withMethod: compiledMethod.
	SystemChangeNotifier uniqueInstance doSilently: 
		[ aClass organization 
			classify: selector
			under: aSymbol ]! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'pmm 7/4/2009 11:19'!
contentsOfFile: aString binary: aBoolean
	| stream |
	stream := aBoolean
		ifTrue: [ (FileDirectory default oldFileNamed: aString)
			binary;
			yourself ]
		ifFalse: [ (MultiByteFileStream oldFileNamed: aString)
			ascii;
			wantsLineEndConversion: true;
			yourself ].
	^[ stream contents ] ensure: [ stream close ]! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'lr 2/4/2009 11:06'!
convertToSmalltalkNewlines: aString
	"Convert any line endings (CR, CRLF, LF) to CR."
	
	aString isNil ifTrue: [ ^ nil ].
	^ String streamContents: [ :writeStream |
		| readStream |
		readStream := aString readStream.
		[ readStream atEnd ] whileFalse: [
			| next |
			next := readStream next.
			next = Character cr
				ifTrue: [
					readStream peek = Character lf
						ifTrue: [ readStream skip: 1 ].
					writeStream nextPut: Character cr ]
				ifFalse: [
					next = Character lf
						ifTrue: [ writeStream nextPut: Character cr ]
						ifFalse: [ writeStream nextPut: next ] ] ] ]! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'jf 1/21/2009 17:31'!
ensureExistenceOfFolder: aString
	"creates a folder named aString in the image directory"
	FileDirectory default assureExistenceOfPath: aString! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'jf 1/21/2009 17:31'!
filesIn: aPathString
	"Return a collection of absolute paths for all the files (no directories) in the directory given by aPathString
	must not include file names that start with ."
	| directory |
	directory := FileDirectory default directoryNamed: aPathString.
	^(directory fileNames
		reject: [ :each | each first = $. ])
		collect: [ :each | directory fullNameFor: each ]! !

!GRPharoPlatform methodsFor: 'processes' stamp: 'jf 2/6/2009 15:58'!
isProcessTerminated: aProcess
	"Return a boolean indicating whether aProcess has been terminated."
	^ aProcess isTerminated! !

!GRPharoPlatform methodsFor: 'version info' stamp: 'jf 2/9/2010 00:56'!
label
	^ 'Pharo'! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'jf 1/21/2009 17:31'!
localNameOf: aFilename
	^(FileDirectory on: aFilename) localName! !

!GRPharoPlatform methodsFor: 'factory' stamp: 'jf 9/30/2009 00:45'!
newRandom
	"Answers the random number generator to be used to create session and continuation keys. Make sure it is seeded. They only methods that will be sent to it are:
	  #nextInt: - should answer a random integer in the interval [1, anInteger]
	  #randomFrom: - should answer a random element from the given collection
	
	Make sure that both methods are safe under heavy concurrent load.
	
	Used by Gemstone/S traditional Randoms which cannot be persisted.
	Used by Squeak to use a secure random when avaiable."
	^ GRPharoRandomProvider! !

!GRPharoPlatform methodsFor: 'exceptions' stamp: 'jf 1/21/2009 17:31'!
openDebuggerOn: anError
	| process |
	process := Processor activeProcess.
	"If we are running in the UI process, we don't want to suspend the active process. The
	error was presumably triggered while stepping in the Debugger. If we simply immediately
	signal an UnhandledError, the debugger will catch this and display the signaling context.
	It isn't perfect or pretty but it works."
	(ProcessBrowser isUIProcess: process)
		ifTrue: [
			UnhandledError signalForException: anError ]
		ifFalse: [
			WorldState addDeferredUIMessage:
				[process
					debug: anError signalerContext
					title: anError description
					full: true].
			process suspend ]! !

!GRPharoPlatform methodsFor: 'factory' stamp: 'dkh 9/4/2009 09:14'!
readWriteByteStream
	"ByteArray based read write stream"
	
	^ ReadWriteStream on: (ByteArray new: 4096)! !

!GRPharoPlatform methodsFor: 'factory' stamp: 'dkh 9/4/2009 09:14'!
readWriteCharacterStream
	"String based read write stream"

	^ ReadWriteStream on: (String new: 4096)! !

!GRPharoPlatform methodsFor: 'startup' stamp: 'jf 1/21/2009 17:31'!
removeFromShutDownList: anObject
	"Remove anObject from the shutdown list in the system."

	Smalltalk removeFromShutDownList: anObject! !

!GRPharoPlatform methodsFor: 'startup' stamp: 'jf 1/21/2009 17:31'!
removeFromStartUpList: anObject
	"Remove anObject from the startup list in the system."

	Smalltalk removeFromStartUpList: anObject! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'jf 1/21/2009 17:31'!
removeSelector: aSymbol from: aClass
	aClass removeSelectorSilently: aSymbol! !

!GRPharoPlatform methodsFor: 'cryptography' stamp: 'jf 1/21/2009 17:31'!
secureHashFor: aString
	^ SecureHashAlgorithm new hashMessage: aString! !

!GRPharoPlatform methodsFor: 'factory' stamp: 'jf 1/21/2009 17:31'!
semaphoreClass
	"used by Gemstone/S traditional Semaphores which cannot be persisted"
	^Semaphore! !

!GRPharoPlatform methodsFor: 'exceptions' stamp: 'jf 1/21/2009 17:31'!
stackDepth

	| depth current |
	depth := 0.
	current := thisContext.
	[ current isNil ] whileFalse: [
		current := current sender.
		depth := depth + 1 ].
	^ depth - 1! !

!GRPharoPlatform methodsFor: 'processes' stamp: 'jf 2/6/2009 15:59'!
terminateProcess: aProcess
	"Permanently terminate the process, unwinding first to execute #ensure: and #ifCurtailed: blocks."
	aProcess terminate! !

!GRPharoPlatform methodsFor: 'private-file library' stamp: 'jf 1/21/2009 17:31'!
useByteArrayLiterals
	"whether ByteArray literals can/should be used"
	^Preferences
		valueOfFlag: #compileUseNewCompiler
		ifAbsent: [ false ]! !

!GRPharoPlatform methodsFor: 'factory' stamp: 'jf 1/21/2009 17:31'!
weakDictionaryOfSize: aNumber
	^IdentityDictionary new: aNumber! !

!GRPharoPlatform methodsFor: 'file library' stamp: 'pmm 7/4/2009 11:19'!
write: aStringOrByteArray toFile: aFileNameString inFolder: aFolderString
	"writes aStringOrByteArray to a file named aFilenameString in the folder aFolderString"
	| folder stream fullFilePath |
	folder := FileDirectory default directoryNamed: aFolderString.
	fullFilePath := folder fullNameFor: aFileNameString.
	stream := aStringOrByteArray isString
		ifTrue: [
			(CrLfFileStream forceNewFileNamed: fullFilePath)
				ascii;
				wantsLineEndConversion: true;
				yourself ]
		ifFalse: [ (FileStream forceNewFileNamed: fullFilePath) binary ].
	[ stream nextPutAll: aStringOrByteArray ]
		ensure: [ stream close ]! !

!Object methodsFor: '*grease-pharo-core' stamp: 'jf 9/30/2009 01:02'!
displayString
	self 
		greaseDeprecatedApi: 'Object>>#displayString'
		details: 'Use #seasideString.'.
	^ self asString! !

!Object methodsFor: '*grease-pharo-core' stamp: 'jf 9/30/2009 01:10'!
greaseString
	^ self asString! !

GRObject subclass: #GRPharoRandomProvider
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Grease-Pharo-Core'!

GRPharoRandomProvider class
	instanceVariableNames: 'mutex generator'!

GRPharoRandomProvider class
	instanceVariableNames: 'mutex generator'!

!GRPharoRandomProvider class methodsFor: 'private' stamp: 'jgf 1/25/2009 12:05'!
initialize
	Smalltalk addToStartUpList: self.
	self startUp! !

!GRPharoRandomProvider class methodsFor: 'public' stamp: 'pmm 1/10/2009 16:07'!
nextInt: anInteger
	"Answer a random integer in the interval [1, anInteger]"
	^mutex critical: [
		generator nextInt: anInteger ]! !

!GRPharoRandomProvider class methodsFor: 'private' stamp: 'pmm 1/10/2009 16:03'!
randomClass
	^Random! !

!GRPharoRandomProvider class methodsFor: 'public' stamp: 'jf 9/25/2009 16:35'!
randomFrom: aCollection
	|random count|
	random := self nextInt: aCollection size.
	^ aCollection isSequenceable
		ifTrue: [ aCollection at: random ]
		ifFalse: [
			count := 1.
			aCollection do: [ :ea |
				count = random ifTrue: [ ^ ea ].
				count := count + 1 ] ]! !

!GRPharoRandomProvider class methodsFor: 'class initialization' stamp: 'pmm 1/10/2009 16:01'!
startUp
	generator := self randomClass new.
	mutex := Semaphore forMutualExclusion! !

!GRPharoRandomProvider class methodsFor: 'private' stamp: 'jf 9/30/2009 00:19'!
unload
	GRPlatform current removeFromStartUpList: self! !

!SocketStream methodsFor: '*grease-pharo-core' stamp: 'pmm 4/11/2010 10:57'!
seasideNext: anInteger putAll: aCollection startingAt: startIndex
	"Put a String or a ByteArray onto the stream starting at the given position.
	Currently a large collection will allocate a large buffer."

	| toPut |
	anInteger = 0 ifTrue: [
		^ aCollection ].
	toPut := binary ifTrue: [ aCollection asByteArray ] ifFalse: [ aCollection asString ].
	self adjustOutBuffer: anInteger.
	outBuffer replaceFrom: outNextToWrite to: outNextToWrite + anInteger with: toPut startingAt: startIndex.
	outNextToWrite := outNextToWrite + anInteger.
	self checkFlush! !

!Number methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 00:12'!
milliseconds
	^ self milliSeconds! !

!WriteStream methodsFor: '*grease-pharo-core' stamp: 'lr 1/22/2009 22:31'!
crlf
	self nextPut: Character cr; nextPut: Character lf! !

!GRSmallDictionary methodsFor: '*grease-pharo-core' stamp: 'pmm 8/29/2009 18:13'!
customizeExplorerContents
	^true! !

!GRSmallDictionary methodsFor: '*grease-pharo-core' stamp: 'pmm 8/29/2009 18:14'!
explorerContents
	| contents |	
	contents := OrderedCollection new.
	self keysAndValuesDo: [ :key :value |
		contents add: (ObjectExplorerWrapper
			with: value
			name: (key printString contractTo: 32)
			model: self) ].
	^contents! !

!GRSmallDictionary methodsFor: '*grease-pharo-core' stamp: 'pmm 8/29/2009 18:15'!
hasContentsInExplorer
	^true! !

!Interval methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 03:25'!
any
	"#first (used by SequenceableCollection>>anyOne) is an accessor of
	Interval and does not error on an empty Interval."
	
	^ self at: 1! !

!PositionableStream methodsFor: '*grease-pharo-core' stamp: 'jf 9/30/2009 01:06'!
greaseUpToAll: aCollection 
	"Needed for Seaside ports to other dialects where #upToAll: may have
	different semantics"
	^ self upToAll: aCollection! !

!Collection methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 00:41'!
any
	^ self anyOne! !

!Collection methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 02:02'!
sorted
	^ self asArray sorted! !

!Collection methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 02:02'!
sorted: sortBlock
	^ self asArray sorted: sortBlock! !

!BlockClosure methodsFor: '*grease-pharo-core' stamp: 'lr 4/8/2009 13:33'!
fixCallbackTemps! !

!BlockClosure methodsFor: '*grease-pharo-core' stamp: 'lr 4/8/2009 13:36'!
valueWithPossibleArguments: anArray
	| args |
	(anArray size == self numArgs)
		ifTrue: [ ^ self valueWithArguments: anArray ].
	args := Array new: self numArgs.
	args replaceFrom: 1
		to: (anArray size min: args size)
		with: anArray
		startingAt: 1.
	^ self valueWithArguments: args! !

!SequenceableCollection methodsFor: '*grease-pharo-core' stamp: 'jok 3/22/2010 11:48'!
beginsWithSubCollection: aSequenceableCollection
	"Some platforms implement #beginsWith: to answer true for an empty argument."
	^ self beginsWith: aSequenceableCollection! !

!SequenceableCollection methodsFor: '*grease-pharo-core' stamp: 'jok 3/22/2010 11:47'!
endsWithSubCollection: aSequenceableCollection
	"Some platforms implement #endsWith: to answer true for an empty argument."
	^ self endsWith: aSequenceableCollection! !

!SequenceableCollection methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 02:01'!
sorted
	^ self sorted: [ :a :b | a <= b ]! !

!SequenceableCollection methodsFor: '*grease-pharo-core' stamp: 'lr 3/31/2010 08:36'!
sorted: sortBlock
	^ self sortBy: sortBlock! !

!ByteArray methodsFor: '*grease-pharo-core' stamp: 'jf 9/30/2009 01:10'!
greaseString
	"ByteArrays should not automatically be converted to Strings. You should use a GRCodec for this."
	^ self printString! !

!GRPackage class methodsFor: '*grease-pharo-core' stamp: 'lr 2/17/2010 15:11'!
greasePharoCore
	^ self new
		name: 'Grease-Pharo-Core';
		addDependency: 'Grease-Core';
		url: #seasideUrl;
		yourself! !

!Duration class methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 00:12'!
milliseconds: anInteger
	^ self milliSeconds: anInteger! !

!Duration methodsFor: '*grease-pharo-core' stamp: 'jf 2/5/2010 23:58'!
asMilliseconds
	^ self asMilliSeconds! !

!Duration methodsFor: '*grease-pharo-core' stamp: 'jf 2/6/2010 00:02'!
milliseconds
	^ nanos quo: NanosInMillisecond! !

GRCodecStream subclass: #GRPharoConverterCodecStream
	instanceVariableNames: 'converter'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Grease-Pharo-Core'!

!GRPharoConverterCodecStream commentStamp: 'pmm 4/11/2010 18:19' prior: 0!
A WAConverterCodecStream is a WACodec stream around a TextConverter. It is always in text mode.

Instance Variables
	converter:		<TextConverter>

converter
	- the TextConverter used to do the encoding conversion
!

!GRPharoConverterCodecStream class methodsFor: 'instance creation' stamp: 'lr 2/7/2009 12:49'!
on: aStream converter: aConverter
	^ self basicNew initalizeOn: aStream converter: aConverter! !

!GRPharoConverterCodecStream methodsFor: 'accessing' stamp: 'lr 2/7/2009 12:57'!
contents
	^ stream contents! !

!GRPharoConverterCodecStream methodsFor: 'actions' stamp: 'lr 2/7/2009 16:11'!
flush
	stream flush! !

!GRPharoConverterCodecStream methodsFor: 'initialization' stamp: 'pmm 4/11/2010 18:18'!
initalizeOn: aStream converter: aConverter
	self initalizeOn: aStream.
	converter := aConverter! !

!GRPharoConverterCodecStream methodsFor: 'streaming' stamp: 'pmm 4/10/2010 13:32'!
next
	| character |
	character := converter nextFromStream: stream.
	^ character isNil
		ifFalse: [ character asCharacter ]! !

!GRPharoConverterCodecStream methodsFor: 'streaming' stamp: 'lr 2/7/2009 12:57'!
next: anInteger
	| writeStream |
	writeStream := WriteStream on: (String new: anInteger).
	anInteger timesRepeat: [
		writeStream nextPut: (self next
			ifNil: [ ^ writeStream contents ]) ].
	^ writeStream contents! !

!GRPharoConverterCodecStream methodsFor: 'streaming' stamp: 'pmm 4/10/2010 13:30'!
nextPut: aCharacter
	converter nextPut: aCharacter asCharacter toStream: stream! !

!GRPharoConverterCodecStream methodsFor: 'streaming' stamp: 'lr 2/7/2009 12:54'!
nextPutAll: aString
	aString asString do: [ :each | self nextPut: each ]! !

!GRPharoConverterCodecStream methodsFor: 'accessing' stamp: 'lr 2/7/2009 14:30'!
size
	^ stream size! !

GRPharoConverterCodecStream subclass: #GRPharoUtf8CodecStream
	instanceVariableNames: ''
	classVariableNames: 'Latin1ToUtf8Encodings Latin1ToUtf8Map'
	poolDictionaries: ''
	category: 'Grease-Pharo-Core'!

!GRPharoUtf8CodecStream commentStamp: 'pmm 2/20/2009 12:27' prior: 0!
A WAUtf8CodecStream is a WACodecStream optimized for UTF-8 performance in the case where most of the characters are ASCII.
!

!GRPharoUtf8CodecStream class methodsFor: 'class initialization' stamp: 'lr 7/16/2009 08:54'!
initialize
	(Smalltalk hasClassNamed: #UTF8TextConverter) "guard for Squeak 37"
		ifFalse: [ ^self ].

	Latin1ToUtf8Map := ByteArray new: 256.
	Latin1ToUtf8Encodings := Array new: 256.
	0 to: 255 do:[ :index |
		| latin1 utf8 |
		latin1 := String with: (Character codePoint: index).
		utf8 := latin1 convertToWithConverter: UTF8TextConverter new.
		latin1 = utf8
			ifTrue:[
				Latin1ToUtf8Map at: index + 1 put: 0 ] "no translation needed"
			ifFalse:[
				Latin1ToUtf8Map at: index + 1 put: 1. "no translation needed"
				Latin1ToUtf8Encodings at: index + 1 put: utf8 ] ]! !

!GRPharoUtf8CodecStream methodsFor: 'streaming' stamp: 'pmm 4/10/2010 13:28'!
crlf
	stream nextPut: Character cr.
	stream nextPut: Character lf! !

!GRPharoUtf8CodecStream methodsFor: 'private' stamp: 'pmm 2/20/2009 11:24'!
encodeDefault: aString
	"Convert the given string from UTF-8 using the fast path if converting to Latin-1"
	1 to: aString size by: 1 do: [ :index |
		converter nextPut: (aString at: index) toStream: stream]! !

!GRPharoUtf8CodecStream methodsFor: 'private' stamp: 'pmm 4/11/2010 10:54'!
encodeFast: aByteString
	"Convert the given string from UTF-8 using the fast path if converting to Latin-1"
	| lastIndex nextIndex |
	lastIndex := 1.
	nextIndex := ByteString findFirstInString: aByteString inSet: Latin1ToUtf8Map startingAt: lastIndex.
	nextIndex = 0 ifTrue: [
		^stream nextPutAll: aByteString ].
	[	stream seasideNext: nextIndex - lastIndex putAll: aByteString startingAt: lastIndex.
		stream nextPutAll: (Latin1ToUtf8Encodings at: (aByteString byteAt: nextIndex) + 1).
		lastIndex := nextIndex + 1.
		nextIndex := ByteString findFirstInString: aByteString inSet: Latin1ToUtf8Map startingAt: lastIndex.
		nextIndex = 0 ] whileFalse.
	stream seasideNext: aByteString size - lastIndex + 1 putAll: aByteString startingAt: lastIndex.! !

!GRPharoUtf8CodecStream methodsFor: 'private' stamp: 'jf 9/30/2009 00:39'!
invalidUtf8
	^ GRInvalidUtf8Error signal: 'Invalid UTF-8 input'! !

!GRPharoUtf8CodecStream methodsFor: 'streaming' stamp: 'pmm 2/17/2010 20:09'!
next
	^ (self next: 1) first! !

!GRPharoUtf8CodecStream methodsFor: 'streaming' stamp: 'lr 7/16/2009 08:53'!
next: anInteger
	"Convert the given string from UTF-8 using the fast path if converting to Latin-1"
	| outStream lastIndex byte1 byte2 byte3 byte4 unicode count |
	lastIndex := 1.
	outStream := WriteStream on: (String new: anInteger).
	count := 0.
	[ count < anInteger and: [ stream atEnd not ] ] whileTrue: [
		byte1 := stream next.
		unicode := byte1.
		(byte1 bitAnd: 16rE0) = 192 ifTrue: [ "two bytes"
			byte2 := stream next.
			(byte2 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			unicode := ((byte1 bitAnd: 31) bitShift: 6) + (byte2 bitAnd: 63)].
		(byte1 bitAnd: 16rF0) = 224 ifTrue: [ "three bytes"
			byte2 := stream next.
			(byte2 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			byte3 := stream next.
			(byte3 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			unicode := ((byte1 bitAnd: 15) bitShift: 12) + ((byte2 bitAnd: 63) bitShift: 6)
				+ (byte3 bitAnd: 63)].
		(byte1 bitAnd: 16rF8) = 240 ifTrue: [ "four bytes"
			byte2 := stream next.
			(byte2 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			byte3 := stream next.
			(byte3 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			byte4 := stream next.
			(byte4 bitAnd: 16rC0) = 16r80 ifFalse: [ self invalidUtf8 ].
			unicode := ((byte1 bitAnd: 16r7) bitShift: 18) +
							((byte2 bitAnd: 63) bitShift: 12) + 
							((byte3 bitAnd: 63) bitShift: 6) +
							(byte4 bitAnd: 63)].
		unicode ifNil: [ self invalidUtf8 ].
		unicode = 16rFEFF "ignore BOM" ifFalse: [
			outStream nextPut: (Character codePoint: unicode).
			count := count + 1 ].
		unicode := nil ].
	^outStream contents! !

!GRPharoUtf8CodecStream methodsFor: 'streaming' stamp: 'lr 2/20/2009 18:35'!
nextPut: aCharacter
	self nextPutAll: (String with: aCharacter)! !

!GRPharoUtf8CodecStream methodsFor: 'streaming' stamp: 'pmm 4/10/2010 13:28'!
nextPutAll: aString
	aString isByteString
		ifTrue: [ self encodeFast: aString ]
		ifFalse: [ self encodeDefault: aString ]! !

!Character methodsFor: '*grease-pharo-core' stamp: 'jf 9/30/2009 01:04'!
greaseInteger
	"Answer an unicode code point of the receiver."
	^ self charCode! !
GRPharoPlatform initialize!
GRPharoRandomProvider initialize!
GRPharoUtf8CodecStream initialize!
